//+------------------------------------------------------------------+
//|                                               PortfolioXpert.mq4 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, TheXpert"
#property link      "theforexpert@gmail.com"

extern int Depth = 100;
extern int Magic = 346548;
extern int Slippage = 30;
extern int TimesToRepeat = 5;
extern double Threshold = 0.5;

string Pairs[] = {"EURUSD", "XAUUSD"};//, "GBPUSD", "USDCAD", "USDCHF"};//, "USDJPY"};
double Koefs[] = {1, 1.5, 2.25, 3.375, 5.0625, 7.59375, 11.390625, -1, -1.5, -2.25, -3.375, -5.0625, -7.59375};

double Points[];
double Quotes[];
double Solution[];

void CloseBy(string s)
{
   int buyTicket = -1;
   int sellTicket = -1;
   
   double buyLot;
   double sellLot;
   
   for(int i = OrdersTotal() - 1; i >= 0; i--)
   {
      // already closed
      if(!OrderSelect(i, SELECT_BY_POS)) continue;
      // not current symbol
      if(OrderSymbol() != s) continue;
      // order was opened in another way
      if(OrderMagicNumber() != Magic) continue;
      
      if(OrderType() == OP_SELL)
      {
         sellTicket = OrderTicket();
         sellLot = OrderLots();
         if (buyTicket != -1) break;
      }
      
      if(OrderType() == OP_BUY)
      {
         buyTicket = OrderTicket();
         buyLot = OrderLots();
         if (sellTicket != -1) break;
      }
   }
   
   if (buyTicket == -1 || sellTicket == -1) return;
   
   if (sellLot > buyLot)
   {
      OrderCloseBy(sellTicket, buyTicket);
   }
   else
   {
      OrderCloseBy(buyTicket, sellTicket);
   }
}

bool LeastSquares(double x[], double y[], double& a, double& b, double& error)
{
   int size = ArraySize(x);
   if (size < 3) return (false);
   if (ArraySize(y) != size) return (false);
   
   double ex = 0; // Sum(x)
   double ey = 0; // Sum(y)
   double exx = 0; // Sum(x*x)
   double exy = 0; // Sum(x*y)

   // counting summaries
   for (int i = 0; i < size; i++)
   {
      ex  += x[i];
      ey  += y[i];
      exx += x[i]*x[i];
      exy += x[i]*y[i];
   }
   
   // y = ax + b
   // counting a and b
   a = (size*exx - ex*ex);
   if (a == 0) return (false);
   
   a = (size*exy - ex*ey)/a;
   b = (ey - a*ex)/size;
   
   error = 0;
   
   for (i = 0; i < size; i++)
   {
      double value = (a*x[i] + b - y[i]);
      error += value*value;
   }

   return(true);
}

datetime LastTime;

int init()
{
   LastTime = 0;
}

int start()
{
   int size = ArraySize(Pairs);

   if (LastTime == Time[0] )
   {
      for (int i = 0; i < size; i++)
      {
         CloseBy(Pairs[i]);
      }
      return (0);
   }

   bool newBar = true;
   
   for (i = 0; i < size; i++)
   {
      newBar = newBar && (Time[0] == iTime(Pairs[i], 0, 0));
   }
   
   if (!newBar) return (0);
   
   LastTime = Time[0];
   
   double a, b;
   
   int start = GetTickCount();
   if (Solve(a, b))
   {
      if (a < 0)
      {
         a = -a;
         b = -b;
      }
      else
      {
         for (i = 0; i < size; i++)
         {
            Solution[i] = -Solution[i];
         }
      }
   
      for (i = 0; i < size; i++)
      {
         Print(Pairs[i], " -- ", Solution[i]);
      }
      Print("Solved in ", GetTickCount() - start, " ms, angle = ", DoubleToStr(a, 8), ", shift = ", DoubleToStr(b, 8));

      if (a > Threshold)
      {
         for (i = 0; i < size; i++)
         {
            Solution[i] = MathRound(Solution[i]*100)/10.0;
         }
      }
      else
      {
         for (i = 0; i < size; i++)
         {
            Solution[i] = 0;
         }
      }
      
      Trade();
   }
   else Print ("Failed to solve");

   return(0);
}

bool Solve(double& angle, double& shift)
{
   int size = ArraySize(Pairs);
   int count = ArraySize(Koefs);
   
   double now[];
   
   double x[];
   
   ArrayResize(Points, size);
   ArrayResize(Quotes, size*Depth);
   ArrayResize(Solution, size);
   ArrayResize(now, size);
   
   ArrayResize(x, Depth);
   for (int i = 0; i < Depth; i++)
   {
      x[i] = i;
   }

   double weights[];
   ArrayResize(weights, size);

   ArrayInitialize(now, 0);
   ArrayInitialize(weights, 0);
   ArrayInitialize(Solution, 0);
   
   for (i = 0; i < size; i++)
   {
      double pt = MarketInfo(Pairs[i], MODE_TICKSIZE);
      if (pt == 0) return (false);
      pt = MarketInfo(Pairs[i], MODE_TICKVALUE)/pt;
      
      for (int j = 0; j < Depth; j++)
      {
         Quotes[Depth*i + j] = iClose(Pairs[i], 0, j + 1);
      }
      
      Points[i] = pt;
   }
   
   double BigBig = 100000000000000000;
   double bestError = BigBig;
   double bestA, bestB;

   while (true)
   {
      double a, b, err;
      double y[];
      
      ArrayResize(y, Depth);
      ArrayInitialize(y, 0);
      
      double sum = 0;
      for (i = 0; i < size; i++)
      {
            int pos = now[i];
         weights[i] = Koefs[pos];
         sum += MathAbs(weights[i]);
      }
      
      for (i = 0; i < size; i++)
      {
         weights[i] /= sum;
      }

         for (i = 0; i < size; i++)
         {
         for (j = 0; j < Depth; j++)
         {
            y[j] += (Quotes[i*Depth + j] - Quotes[i*Depth])*Points[i]*weights[i];
         }
         }
         
      if (LeastSquares(x, y, a, b, err))
      {
         if (err < bestError && MathAbs(a) > Threshold && a*b < 0)
         {
            bestA = a;
            bestB = b;
            bestError = err;
            
            ArrayCopy(Solution, weights);
         }
      }
      
      if(!Next(now, count)) break;
   }
   
   angle = bestA;
   shift = bestB;
   return (bestError != BigBig);
}

bool Next(double& array[], int count)
{
   int size = ArraySize(array);
   
   double old = array[0];
   
   array[size - 1] += 1;
   for (int i = size - 1; i > 0; i--)
   {
      if (array[i] == count)
      {
         array[i - 1] += 1;
         array[i] = 0;
      }
   }
   if (array[0] == count) return (false);
   if (array[0] != old)
   {
      //Print(array[0], " of ", count, " done.");
   }
   return (true);
}

void Trade()
{
   int size = ArraySize(Pairs);
   
   for (int i = 0; i < size; i++)
   {
      CorrectPosition(Pairs[i], Solution[i]);
   }
}

void CorrectPosition(string s, double lot)
{
   double total = 0;
   for(int i = OrdersTotal() - 1; i >= 0; i--)
   {
      // already closed
      if(!OrderSelect(i, SELECT_BY_POS)) continue;
      // not current symbol
      if(OrderSymbol() != s) continue;
      // order was opened in another way
      if(OrderMagicNumber() != Magic) continue;
      
      if(OrderType() == OP_SELL) total -= OrderLots();
      if(OrderType() == OP_BUY) total += OrderLots();
   }
   
   total = lot - total;
   
   if (total > 0)
   {
      OpenBuy(s, Magic, total);
   }
   else if (total < 0)
   {
      OpenSell(s, Magic, -total);
   }
}

void OpenBuy(string s, int MN, double Lot)
{
   int count = 0;
   while (count < TimesToRepeat)
   {
      WaitForContext();
      double ask = MarketInfo(s, MODE_ASK);
      
      int res = OrderSend(s, OP_BUY, Lot, ask, Slippage, 0, 0, NULL, MN, 0);
      
      if (res > 0) return;

      count++;
   }
}

void OpenSell(string s, int MN, double Lot)
{
   int count = 0;
   while (count < TimesToRepeat)
   {
      WaitForContext();
      double bid = MarketInfo(s, MODE_BID);
   
      int res = OrderSend(s, OP_SELL, Lot, bid, Slippage, 0, 0, NULL, MN, 0);
      
      if (res > 0) return;

      count++;
   }
}

void WaitForContext()
{
   while (IsTradeContextBusy())
   {
      Sleep(100);
   }
}

